/*
 * FabMotorController.cpp v0.1
 * A library to control through I2C the FabMotorController, 
 * a hardware based on the ATTINY1614 and the L298N driver.
 * 
 * Author: Harley Lara
 * Create: 01 May 2021
 * License: (CC BY-SA 4.0) Attribution-ShareAlike 4.0 International
 * 
 * This work may be reproduced, modified, distributed,
 * performed, and displayed for any purpose, but must
 * acknowledge this project. Copyright is retained and
 * must be preserved. The work is provided as is; no
 * warranty is provided, and users accept all liability.
 * 
 */

#include "FabMotorController.h"
#include <Wire.h>

/******** Initializes and sets the I2C address ********/
// addr: motor controller address (default 0x04)
int FabMotorController::begin(unsigned char addr){
  if (addr != 0x04){
    Serial.println("[ERROR]: Incorrect I2C address, default address 0x04.");
    return -1; // Incorrect I2C address
  }
  Wire.begin();
  delayMicroseconds(10000);
  this -> _addr = addr;
  Serial.println("[INFO]: I2C Initialization");
  return 0; // Initialization completed without errors
}

/********  Activates the output at a given speed ********/
// output: OUTPUT1, OUTPUT2
// _speed: from -100 to 100, 
//         where _speed greater than zero moves the motor clockwise 
//         (the direction of the motor depends on the physical connection 
//         of the motor, if it turns counterclockwise it is necessary to 
//         reverse the connection).
void FabMotorController::run(unsigned char output, int _speed){
  if (output < OUTPUT1 || output > OUTPUT2){
    Serial.println("[ERROR]: Out of range, must be OUPUT1 or OUPUT2, or alternatively 1 or 2");
    return;
  }
  if (output == OUTPUT1){
    if (_speed >= 0){
      this->_OUT1_direction = 1;
      _speed = _speed > 100 ? 100 : _speed;
      this->_speed1 = map(_speed, 0, 100, 0, 255);
    } else if(_speed < 0){
      this->_OUT1_direction = 0;
      
      _speed = _speed < -100 ? 100 : -(_speed);
      this->_speed1 = map(_speed, 0, 100, 0, 255);
    }

    // Send action OUTPUT 1
    Serial.println("[INFO]: Sending Command to OUPUT1");
    Wire.beginTransmission(this -> _addr);  // Starts I2C transmission
    Serial.println(_OUT1_direction);
    Wire.write(OUTPUT1);                          // Set the output to be activated
    Wire.write(this->_speed1);              // Set the speed
    Wire.write(this->_OUT1_direction);    // Set the direction
    Wire.endTransmission();                 // Transmission stops
    delay(4);
    Serial.println("[INFO]: Command sent");
     
  }else if(output == OUTPUT2){
      if (_speed >= 0){
      this -> _OUT2_direction = 1;
      _speed = _speed > 100 ? 100 : _speed;
      this -> _speed2 = map(_speed, 0, 100, 0, 255);
    } else if(_speed < 0){
      this -> _OUT2_direction = 0;
      _speed = _speed < -100 ? 100 : -(_speed);
      this->_speed2 = map(_speed, 0, 100, 0, 255);
    }

    // Send action OUTPUT 2
    Serial.println("[INFO]: Sending Command to OUPUT2");
    Wire.beginTransmission(this -> _addr);  // Starts I2C transmission
    Wire.write(OUTPUT2);                          // Set the output to be activated
    Wire.write(this->_speed2);              // Set the speed
    Wire.write(this -> _OUT2_direction);    // Set the direction
    Wire.endTransmission();                 // Transmission stops
    delay(4);
    Serial.println("[INFO]: Command sent");
    
  }
}

void FabMotorController::stop(unsigned char output){
  if (output < OUTPUT1 || output > OUTPUT2){
    Serial.println("[ERROR]: Out of range, must be OUPUT1 or OUPUT2, or alternatively 1 or 2");
    return;
  }

  run(output, 0);
}
